==> satellite-4.2.2.tar.gz: satellite-4.2.2/modules/gpm/command/gsolm.cpp <==
/* 
 * Copyright (c) 2003 RIKEN (The Institute of Physical and Chemical Research)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY RIKEN AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL RIKEN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $SATELLITE: satellite4/modules/gpm/command/gsolm.cpp,v 1.6 2004/09/09 10:45:13 orrisroot Exp $ */

/************************************************
*                                               *
*        3-D GRAPHIC COMMAND                    *
*                                               *
*************************************************
*                                               *
* GSOLM IFIL,RHV1,RHV2,IFUC1,IFUC2,IFUC3,IFUC4, *
*       ICLR                                    *
*       IFIL        : FILE NUMBER               *
*       RHV1 , RHV2 : AREA TYPE                 *
*       IFUC1 , IFUC2 , IFUC3 , IFUC4           *
*                                               *
*                                               *
*     SATELITE Version          90/08/03        *
*            C Version          90/08/25        *
*       Buffer Version          90/08/30        *
*    SATELLITE Version          94/03/07        *
*                                               *
*               Coded By  T.Kobayashi           *
*            Modified By  K.Takebe              *
*            Modified By  H.Ashida              *
*                                               *
************************************************/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "SL_macro.h"
#include "SL_cmd.h"

#include "libgpm.h"
#include "gsolm_sub.h"
#include "ldata.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef M_PI
# define M_PI 3.14159265358979323846
#endif

static void isort(float **a, int nx, int ny);

DLLEXPORT int mod_gpm_gsolm(){
  static char def_axis[2] = "X";
  int   index[MAX_INDEX];
  int   bufno, ifunc[4], icol, xdiv, ydiv;
  int   i, j, nx, ny, mx, my, viewpoint, mode;
  int   **ia;
  float rhv[2], amxn[2], xypl[2][2], **a;
  char  *timeaxis;

  float x1 = 0.0, x2 = 0.0, x3 = 0.0, x4 = 0.0;
  float y1 = 0.0, y2 = 0.0, y3 = 0.0, y4 = 0.0;
  float rhx, rhy, axleng, ayleng, azleng, axorg, ayarg, fontsize;
  float xMin, xMax, yMin, yMax, zMax, zMin;
  int   apy, apz;
  
  /* LOAD SYSTEM PARAMETER */
  regpm();
  
  bufno = GetBufferID(0);
  if(GetBufferInfo(bufno, index) != 2){
    return 14; /* data dimension mismatch */
  }

  rhv[0]    = (float)GetScalar(1);
  rhv[1]    = (float)GetScalar(2);
  ifunc[0]  = (int) GetScalar(3); /* hide line */
  ifunc[1]  = (int) GetScalar(4);
  ifunc[2]  = (int) GetScalar(5);
  ifunc[3]  = (int) GetScalar(6);
  icol      = (int) GetScalar(7);
  xdiv      = (int) GetScalar(8);
  ydiv      = (int) GetScalar(9);
  timeaxis  =       GetString(10);
  viewpoint = ((int)GetScalar(11) > 0) ? 1 : -1;
  mode      = ((int)GetScalar(12) > 0) ? 1 : 0;
  fontsize  = (float)GetScalar(13);

  if(timeaxis == NULL)
    timeaxis = def_axis;

  if(fontsize <= 0.0)
    fontsize = 3.5;

  xypl[0][0] = GpmCont.xOrigin;
  xypl[0][1] = GpmCont.yOrigin;
  xypl[1][0] = GpmCont.xSize;
  xypl[1][1] = GpmCont.ySize;

  /* Load Data */
  if(*timeaxis == 'X' || *timeaxis == 'x'){
    nx = index[0];
    ny = index[1];
  }else{
    nx = index[1];
    ny = index[0];
  }

  a = gpmcmd_ldata_ldata(bufno, nx, ny, timeaxis, 1);
  if(a == NULL)
    return 15; /* buffer read error */

  if(xdiv < 1) xdiv = 1;
  if(ydiv < 1) ydiv = 1;
  gpmcmd_ldata_thinout(a, &nx, &ny, xdiv, ydiv);
  if(viewpoint > 0)
    isort(a, nx, ny);
  
  mx = nx - 1;
  my = ny - 1;

  if(GpmCont.xMode == 0){
    GpmCont.xMin = xMin  = 0.0f;
    GpmCont.xMax = xMax  = (float)my;
  }else{
    xMin  = GpmCont.xMin;
    xMax  = GpmCont.xMax;
/*
    if(GpmCont.xType == 1){
      xMin = (xMin > 0.0) ? log10(xMin) : 0;
      xMax = (xMax > 0.0) ? log10(xMax) : my;
    }
*/
  }

  if(GpmCont.yMode == 0){
    GpmCont.yMin = yMin  = 0.0f;
    GpmCont.yMax = yMax  = (float)mx;
  }else{
    yMin  = GpmCont.yMin;
    yMax  = GpmCont.yMax;
/*
    if(GpmCont.yType == 1){
      yMin = (yMin > 0.0) ? log10(yMin) : 0;
      yMax = (yMax > 0.0) ? log10(yMax) : mx;
    }
*/
  }
  GpmCont.axisType = 0;
  
  zMin = a[0][0];
  zMax = a[0][0];
  
  for(i = 0; i < nx; i++)
    for(j = 0; j < ny; j++){
      if(a[i][j] > zMax) zMax = a[i][j];
      if(a[i][j] < zMin) zMin = a[i][j];
    }

  if(GpmCont.zMode == 0){
    GpmCont.zMin = amxn[0] = zMin;
    GpmCont.zMax = amxn[1] = zMax;
  }else{
    amxn[0] = GpmCont.zMin;
    amxn[1] = GpmCont.zMax;
/*
    if(GpmCont.zType == 1){
      amxn[0] = (amxn[0] > 0.0) ? log10(amxn[0]) : -1.0;
      amxn[1] = (amxn[1] > 0.0) ? log10(amxn[1]) :  0.0;
    }
*/
  }

  printf("  Matrix Size  %d x %d\n", nx, ny);
  printf("  (scale: min = %g, max = %g, data: min = %g, max = %g)\n", 
         amxn[0], amxn[1], zMin, zMax);
  
  if(gopen(GpmCont.paper, GpmCont.orientation, GpmCont.device, 
           GpmCont.paperWidth, GpmCont.paperHeight) != 0)
    return 6; /* window does not exist */

  ia = (int**)gpmcmd_ldata_malloc2d(nx, ny, sizeof(int));

  if(checktodvi() != 0)
    libgpm_dvi_write_param(&gpmdev_cont.dvi, GpmCont.factor, 
                           GpmCont.xOrigin, GpmCont.yOrigin);

  gfactor(GpmCont.factor);
  gfont(GpmCont.fontType);
  gorigin(GpmCont.xOrigin, GpmCont.yOrigin);
  
  /* 3D Frame Draw */
  if(ifunc[3] < 0 || ifunc[3] > 10)
    ifunc[3] = -1;
  if(ifunc[3] != -1){
    gpen(GpmCont.fLineWidth, GpmCont.fLineType, 0);
    gnewpen(ifunc[3]);
    rhx = (float)fabs((double)rhv[0]);
    rhy = (float)fabs((double)rhv[1]);
    y1 = 0.0;
    y2 = xypl[1][1] * rhy;
    y3 = xypl[1][1] * (1.0f - rhy);
    y4 = xypl[1][1];
    
    if(rhv[0] * rhv[1] > 0){
      x1 = 0.0;
      x2 = xypl[1][0] * rhx;
      x3 = xypl[1][0] * (1.0f - rhx);
      x4 = xypl[1][0];
      
      axorg = 0.0;
      ayarg = (float)(fabs(atan((double)(y2-y1)/(double)(x4-x3))));
      apy = 1;
      apz = 0;
    }else{
      x4 = 0.0;
      x3 = xypl[1][0] * rhx;
      x2 = xypl[1][0] * (1.0f - rhx);
      x1 = xypl[1][0];
      
      axorg = xypl[1][0] * rhx;
      ayarg = (float)(M_PI-fabs(atan((double)(y2-y1)/(double)(x4-x3))));
      apy = 0;
      apz = 1;
    }
    gplot(x1, y1, 0);
    gplot(x1, y3, -1);
    gplot(x2, y2, -1);
    gplot(x3, y1, -2);
    
    gplot(x4, y2, 0);
    gplot(x4, y4, -1);
    gplot(x2, y2, -1);
    gplot(x3, y1, -2);
    
    gplot(x2, y4, 0);
    gplot(x2, y2, -1);
    gplot(x1, y3, -1);
    gplot(x4, y4, -2);
    
    /* Draw ZERO Level */
    if(icol < 0 || icol > 10)
      icol = -1;
    if(icol != -1){
      /* ifunc[0] = 1; */
      for(i = 0; i < nx; i++)
        for(j = 0; j < ny; j++)
          a[i][j] = 0.0;
      gorigin(GpmCont.xOrigin, GpmCont.yOrigin);
      gpen(GpmCont.gLineWidth, GpmCont.gLineType, 0);
      gpmcmd_gsolm_solm0r(a, ia, nx, ny, mx, my, amxn, rhv, ifunc, xypl,
                          GpmCont.gColor, icol, mode);

      /* not good code ! */
      efree(a); a = NULL;
      a = gpmcmd_ldata_ldata(bufno, nx, ny, timeaxis, 1);
      if(a == NULL)
        return 15; /* buffer read error */

      if(xdiv < 1) xdiv = 1;
      if(ydiv < 1) ydiv = 1;
      gpmcmd_ldata_thinout(a, &nx, &ny, xdiv, ydiv);
      if(viewpoint > 0)
        isort(a, nx, ny);
    }

    axleng = xypl[1][0] * (1.0f - rhx);
    ayleng = (float)sqrt((double)((x4 - x3) * (x4 - x3)
                                    + (y2 - y1) * (y2 - y1)));
    azleng = xypl[1][1] * (1.0f - rhy);
    
    gpen(GpmCont.fLineWidth, GpmCont.fLineType, 0);
    gnewpen(ifunc[3]);

    /* X Axis */
    gaxis(axorg, 0.0, axleng, (float)0, xMin, xMax,
          0.0, 2, 1, GpmCont.title_x[0], fontsize, 0, 0.0);

    /* Y Axis */
    gaxis(x3, 0.0, ayleng, (float)ayarg, yMin, yMax,
          0.0, 2, apy, GpmCont.title_y[0], fontsize, 0, 0.0);
    /* Z Axis */
    gaxis(x1, 0.0, azleng, (float)(M_PI/2.0), amxn[0], amxn[1],
          0.0, 1, apz, GpmCont.title_z[0], fontsize, 0, 0.0);
  }

  gpen(GpmCont.gLineWidth, GpmCont.gLineType, 0);
  gnewpen(GpmCont.gColor);
  gpmcmd_gsolm_solm0r(a, ia, nx, ny, mx, my, amxn, rhv, ifunc, xypl,
                      GpmCont.gColor, GpmCont.fColor, mode);
  
  if(ifunc[3] != -1){
    gpen(GpmCont.fLineWidth, GpmCont.fLineType, 0);
    gnewpen(ifunc[3]);
    gplot(x3, y3, 0);
    gplot(x3, y1, -1);
    gplot(x1, y3, -1);
    gplot(x4, y4, -2);
  }
  gflush();
  gclose();
  wrgpm();

  return 0;
}

static void isort(float **a, int nx, int ny){
  register int i;
  int          half = nx / 2;
  float       *tmp;
  for(i = 0; i < half; i++){
    tmp = a[i];
    a[i] = a[nx - i - 1];        
    a[nx - i - 1] = tmp;
  }
}

#ifdef __cplusplus
}
#endif

==> satellite-4.2.3.tar.gz: satellite-4.2.3/modules/gpm/command/gsolm.cpp <==
/* 
 * Copyright (c) 2003-2005 RIKEN Japan, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY RIKEN AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL RIKEN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $SATELLITE: satellite4/modules/gpm/command/gsolm.cpp,v 1.9 2005/04/06 08:42:15 orrisroot Exp $ */

/************************************************
*                                               *
*        3-D GRAPHIC COMMAND                    *
*                                               *
*************************************************
*                                               *
* GSOLM IFIL,RHV1,RHV2,IFUC1,IFUC2,IFUC3,IFUC4, *
*       ICLR                                    *
*       IFIL        : FILE NUMBER               *
*       RHV1 , RHV2 : AREA TYPE                 *
*       IFUC1 , IFUC2 , IFUC3 , IFUC4           *
*                                               *
*                                               *
*     SATELITE Version          90/08/03        *
*            C Version          90/08/25        *
*       Buffer Version          90/08/30        *
*    SATELLITE Version          94/03/07        *
*                                               *
*               Coded By  T.Kobayashi           *
*            Modified By  K.Takebe              *
*            Modified By  H.Ashida              *
*                                               *
************************************************/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "SL_macro.h"
#include "SL_cmd.h"

#include "libgpm.h"
#include "gsolm_sub.h"
#include "ldata.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef M_PI
# define M_PI 3.14159265358979323846
#endif

static void isort(float **a, int nx, int ny);

DLLEXPORT int mod_gpm_gsolm(){
  static char def_axis[2] = "X";
  int   index[MAX_INDEX];
  int   bufno, ifunc[4], icol, xdiv, ydiv;
  int   i, j, nx, ny, mx, my, viewpoint, mode;
  int   **ia;
  float rhv[2], amxn[2], xypl[2][2], **a;
  char  *timeaxis;

  float x1 = 0.0, x2 = 0.0, x3 = 0.0, x4 = 0.0;
  float y1 = 0.0, y2 = 0.0, y3 = 0.0, y4 = 0.0;
  float rhx, rhy, axleng, ayleng, azleng, axorg, ayarg, fontsize;
  float xMin, xMax, yMin, yMax, zMax, zMin;
  int   apy, apz;
  gpm_cont_t gpm_cont;
  
  /* LOAD SYSTEM PARAMETER */
  regpm(&gpm_cont);
  
  bufno = GetBufferID(0);
  if(GetBufferInfo(bufno, index) != 2){
    return 14; /* data dimension mismatch */
  }

  rhv[0]    = (float)GetScalar(1);
  rhv[1]    = (float)GetScalar(2);
  ifunc[0]  = (int) GetScalar(3); /* hide line */
  ifunc[1]  = (int) GetScalar(4);
  ifunc[2]  = (int) GetScalar(5);
  ifunc[3]  = (int) GetScalar(6);
  icol      = (int) GetScalar(7);
  xdiv      = (int) GetScalar(8);
  ydiv      = (int) GetScalar(9);
  timeaxis  =       GetString(10);
  viewpoint = ((int)GetScalar(11) > 0) ? 1 : -1;
  mode      = ((int)GetScalar(12) > 0) ? 1 : 0;
  fontsize  = (float)GetScalar(13);

  if(timeaxis == NULL)
    timeaxis = def_axis;

  if(fontsize <= 0.0)
    fontsize = 3.5;

  xypl[0][0] = gpm_cont.xOrigin;
  xypl[0][1] = gpm_cont.yOrigin;
  xypl[1][0] = gpm_cont.xSize;
  xypl[1][1] = gpm_cont.ySize;

  /* Load Data */
  if(*timeaxis == 'X' || *timeaxis == 'x'){
    nx = index[0];
    ny = index[1];
  }else{
    nx = index[1];
    ny = index[0];
  }

  a = gpmcmd_ldata_ldata(bufno, nx, ny, timeaxis, 1);
  if(a == NULL)
    return 15; /* buffer read error */

  if(xdiv < 1) xdiv = 1;
  if(ydiv < 1) ydiv = 1;
  gpmcmd_ldata_thinout(a, &nx, &ny, xdiv, ydiv);
  if(viewpoint > 0)
    isort(a, nx, ny);
  
  mx = nx - 1;
  my = ny - 1;

  if(gpm_cont.xMode == 0){
    gpm_cont.xMin = xMin  = 0.0f;
    gpm_cont.xMax = xMax  = (float)my;
  }else{
    xMin  = gpm_cont.xMin;
    xMax  = gpm_cont.xMax;
/*
    if(gpm_cont.xType == 1){
      xMin = (xMin > 0.0) ? log10(xMin) : 0;
      xMax = (xMax > 0.0) ? log10(xMax) : my;
    }
*/
  }

  if(gpm_cont.yMode == 0){
    gpm_cont.yMin = yMin  = 0.0f;
    gpm_cont.yMax = yMax  = (float)mx;
  }else{
    yMin  = gpm_cont.yMin;
    yMax  = gpm_cont.yMax;
/*
    if(gpm_cont.yType == 1){
      yMin = (yMin > 0.0) ? log10(yMin) : 0;
      yMax = (yMax > 0.0) ? log10(yMax) : mx;
    }
*/
  }
  gpm_cont.axisType = 0;
  
  zMin = a[0][0];
  zMax = a[0][0];
  
  for(i = 0; i < nx; i++)
    for(j = 0; j < ny; j++){
      if(a[i][j] > zMax) zMax = a[i][j];
      if(a[i][j] < zMin) zMin = a[i][j];
    }

  if(gpm_cont.zMode == 0){
    gpm_cont.zMin = amxn[0] = zMin;
    gpm_cont.zMax = amxn[1] = zMax;
  }else{
    amxn[0] = gpm_cont.zMin;
    amxn[1] = gpm_cont.zMax;
/*
    if(gpm_cont.zType == 1){
      amxn[0] = (amxn[0] > 0.0) ? log10(amxn[0]) : -1.0;
      amxn[1] = (amxn[1] > 0.0) ? log10(amxn[1]) :  0.0;
    }
*/
  }

  printf("  Matrix Size  %d x %d\n", nx, ny);
  printf("  (scale: min = %g, max = %g, data: min = %g, max = %g)\n", 
         amxn[0], amxn[1], zMin, zMax);
  
  if(gopen(&gpm_cont, gpm_cont.paper, gpm_cont.orientation, gpm_cont.device, 
           gpm_cont.paperWidth, gpm_cont.paperHeight) != 0)
    return 6; /* window does not exist */

  ia = (int**)gpmcmd_ldata_malloc2d(nx, ny, sizeof(int));

  if(checktodvi(&gpm_cont) != 0)
    libgpm_dvi_write_param(&gpmdev_cont.dvi, gpm_cont.factor, 
                           gpm_cont.xOrigin, gpm_cont.yOrigin);

  gfactor(gpm_cont.factor);
  gfont(gpm_cont.fontType);
  gorigin(gpm_cont.xOrigin, gpm_cont.yOrigin);
  
  /* 3D Frame Draw */
  if(ifunc[3] < 0 || ifunc[3] > 10)
    ifunc[3] = -1;
  if(ifunc[3] != -1){
    gpen(gpm_cont.fLineWidth, gpm_cont.fLineType, 0);
    gnewpen(ifunc[3]);
    rhx = (float)fabs((double)rhv[0]);
    rhy = (float)fabs((double)rhv[1]);
    y1 = 0.0;
    y2 = xypl[1][1] * rhy;
    y3 = xypl[1][1] * (1.0f - rhy);
    y4 = xypl[1][1];
    
    if(rhv[0] * rhv[1] > 0){
      x1 = 0.0;
      x2 = xypl[1][0] * rhx;
      x3 = xypl[1][0] * (1.0f - rhx);
      x4 = xypl[1][0];
      
      axorg = 0.0;
      ayarg = (float)(fabs(atan((double)(y2-y1)/(double)(x4-x3))));
      apy = 1;
      apz = 0;
    }else{
      x4 = 0.0;
      x3 = xypl[1][0] * rhx;
      x2 = xypl[1][0] * (1.0f - rhx);
      x1 = xypl[1][0];
      
      axorg = xypl[1][0] * rhx;
      ayarg = (float)(M_PI-fabs(atan((double)(y2-y1)/(double)(x4-x3))));
      apy = 0;
      apz = 1;
    }
    gplot(x1, y1, 0);
    gplot(x1, y3, -1);
    gplot(x2, y2, -1);
    gplot(x3, y1, -2);
    
    gplot(x4, y2, 0);
    gplot(x4, y4, -1);
    gplot(x2, y2, -1);
    gplot(x3, y1, -2);
    
    gplot(x2, y4, 0);
    gplot(x2, y2, -1);
    gplot(x1, y3, -1);
    gplot(x4, y4, -2);
    
    /* Draw ZERO Level */
    if(icol < 0 || icol > 10)
      icol = -1;
    if(icol != -1){
      /* ifunc[0] = 1; */
      for(i = 0; i < nx; i++)
        for(j = 0; j < ny; j++)
          a[i][j] = 0.0;
      gorigin(gpm_cont.xOrigin, gpm_cont.yOrigin);
      gpen(gpm_cont.gLineWidth, gpm_cont.gLineType, 0);
      gpmcmd_gsolm_solm0r(a, ia, nx, ny, mx, my, amxn, rhv, ifunc, xypl,
                          gpm_cont.gColor, icol, mode);

      /* not good code ! */
      efree(a); a = NULL;
      a = gpmcmd_ldata_ldata(bufno, nx, ny, timeaxis, 1);
      if(a == NULL)
        return 15; /* buffer read error */

      if(xdiv < 1) xdiv = 1;
      if(ydiv < 1) ydiv = 1;
      gpmcmd_ldata_thinout(a, &nx, &ny, xdiv, ydiv);
      if(viewpoint > 0)
        isort(a, nx, ny);
    }

    axleng = xypl[1][0] * (1.0f - rhx);
    ayleng = (float)sqrt((double)((x4 - x3) * (x4 - x3)
                                    + (y2 - y1) * (y2 - y1)));
    azleng = xypl[1][1] * (1.0f - rhy);
    
    gpen(gpm_cont.fLineWidth, gpm_cont.fLineType, 0);
    gnewpen(ifunc[3]);

    /* X Axis */
    gaxis(axorg, 0.0, axleng, (float)0, xMin, xMax,
          0.0, 2, 1, gpm_cont.title_x[0], fontsize, 0, 0.0);

    /* Y Axis */
    gaxis(x3, 0.0, ayleng, (float)ayarg, yMin, yMax,
          0.0, 2, apy, gpm_cont.title_y[0], fontsize, 0, 0.0);
    /* Z Axis */
    gaxis(x1, 0.0, azleng, (float)(M_PI/2.0), amxn[0], amxn[1],
          0.0, 1, apz, gpm_cont.title_z[0], fontsize, 0, 0.0);
  }

  gpen(gpm_cont.gLineWidth, gpm_cont.gLineType, 0);
  gnewpen(gpm_cont.gColor);
  gpmcmd_gsolm_solm0r(a, ia, nx, ny, mx, my, amxn, rhv, ifunc, xypl,
                      gpm_cont.gColor, gpm_cont.fColor, mode);
  
  if(ifunc[3] != -1){
    gpen(gpm_cont.fLineWidth, gpm_cont.fLineType, 0);
    gnewpen(ifunc[3]);
    gplot(x3, y3, 0);
    gplot(x3, y1, -1);
    gplot(x1, y3, -1);
    gplot(x4, y4, -2);
  }
  gflush();
  gclose();
  wrgpm(&gpm_cont);

  return 0;
}

static void isort(float **a, int nx, int ny){
  register int i;
  int          half = nx / 2;
  float       *tmp;
  for(i = 0; i < half; i++){
    tmp = a[i];
    a[i] = a[nx - i - 1];        
    a[nx - i - 1] = tmp;
  }
}

#ifdef __cplusplus
}
#endif
